import React, { useReducer, useState, useEffect, useCallback } from 'react';
import QuizContext from './context';
import reducer, { initialState } from './reducer';
import { QuestionTypes } from './actions';
import QuizQuestions from '../QuizQuestions';
import ResultContainer from '../ResultContainer/ResultContainer';
import { RequestStates } from '../../constants';
import { getNextQuestion, getQuizResults } from '../../utils';
import Spinner from '../Spinner/Spinner';
import useCioClient from '../../hooks/useCioClient';
export default function CioQuiz(props) {
    const { quizId, apiKey, cioJsClient, resultsPageOptions, quizVersionId: quizVersionIdProp, } = props;
    if (!quizId) {
        // eslint-disable-next-line no-console
        console.error('quizId is a required field of type string');
    }
    if (!resultsPageOptions || Object.keys(resultsPageOptions).length === 0) {
        // eslint-disable-next-line no-console
        console.error('resultsPageOptions is a required field of type object');
    }
    if (resultsPageOptions && !resultsPageOptions?.addToCartCallback) {
        // eslint-disable-next-line no-console
        console.error('resultsPageOptions.addToCartCallback is a required field of type function');
    }
    const cioClient = useCioClient({ apiKey, cioJsClient });
    const [state, dispatch] = useReducer(reducer, initialState);
    const [requestState, setRequestState] = useState(RequestStates.Stale);
    const [questionResponse, setQuestionResponse] = useState();
    const [resultsResponse, setResultsResponse] = useState();
    const [firstQuestion, setFirstQuestion] = useState();
    const [quizVersionId, setQuizVersionId] = useState(quizVersionIdProp || '');
    const [quizSessionId, setQuizSessionId] = useState('');
    const isFirstQuestion = firstQuestion?.next_question.id === questionResponse?.next_question.id;
    const quizNextHandler = useCallback((action) => {
        if (action) {
            dispatch(action);
        }
    }, [dispatch]);
    const quizBackHandler = useCallback(() => {
        if (dispatch) {
            dispatch({ type: QuestionTypes.Back });
        }
    }, [dispatch]);
    const contextValue = {
        dispatch,
        questionResponse,
        state,
        resultsResponse,
        isFirstQuestion,
        quizNextHandler,
        quizBackHandler,
        cioClient,
    };
    useEffect(() => {
        (async () => {
            if (cioClient) {
                setRequestState(RequestStates.Loading);
                if (state.isLastAnswer) {
                    try {
                        const quizResults = await getQuizResults(cioClient, quizId, {
                            answers: state.answers,
                            resultsPerPage: resultsPageOptions?.numResultsToDisplay,
                            quizVersionId,
                            quizSessionId,
                        });
                        setResultsResponse(quizResults);
                        setRequestState(RequestStates.Success);
                        setQuestionResponse(undefined);
                    }
                    catch (error) {
                        setResultsResponse(undefined);
                        setRequestState(RequestStates.Error);
                    }
                }
                else {
                    try {
                        const questionResult = await getNextQuestion(cioClient, quizId, {
                            answers: state.answers,
                            quizVersionId,
                            quizSessionId,
                        });
                        setQuestionResponse(questionResult);
                        setRequestState(RequestStates.Success);
                        setResultsResponse(undefined);
                        if (!quizVersionId && questionResult?.quiz_version_id) {
                            setQuizVersionId(questionResult.quiz_version_id);
                        }
                        if (!quizSessionId && questionResult?.quiz_session_id) {
                            setQuizSessionId(questionResult.quiz_session_id);
                        }
                    }
                    catch (error) {
                        setRequestState(RequestStates.Error);
                    }
                }
            }
        })();
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [cioClient, state, quizId, state.isLastAnswer, resultsPageOptions?.numResultsToDisplay]);
    useEffect(() => {
        if (!firstQuestion) {
            setFirstQuestion(questionResponse);
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [questionResponse]);
    const resetQuizSessionId = () => {
        setQuizSessionId('');
    };
    if (requestState === RequestStates.Loading) {
        return (React.createElement("div", { className: 'cio-quiz' },
            React.createElement(Spinner, null)));
    }
    if (requestState === RequestStates.Success) {
        return (React.createElement("div", { className: 'cio-quiz' },
            React.createElement(QuizContext.Provider, { value: contextValue },
                resultsResponse && (React.createElement(ResultContainer, { options: resultsPageOptions, resetQuizSessionId: resetQuizSessionId })),
                questionResponse && React.createElement(QuizQuestions, { questionResponse: questionResponse }))));
    }
    return null;
}
